//	BitArray.h
//	BitArray procedures for RC1
//	version 2.0 [02.06.2000]
//	Copyright by SkyLord [Team 16]

#ifndef __BitArray_h
#define __BitArray_h

#include "DynArray.h"

///////////////////////////////////////////////////////////
//   X:
//   X -  ,   
//        Number
//   {if(Number=0) X=1; else X=LOG2(Number);}
///////////////////////////////////////////////////////////
BYTE BitsPerNumber(DWORD Number)
{
	BYTE result;

  	__asm
   {
		mov ebx, Number
      mov al, 0
BPN_Start:
		shr ebx, 1
      inc al
      cmp ebx, 0
      jne BPN_Start
// if((Number>>al)>0) jump BPN_Start
      mov result, al
   }
   return result;
}
///////////////////////////////////////////////////////////
// BitArray
///////////////////////////////////////////////////////////
class BitArray
{
public:
	ByteArray Bits;
	BitArray() {Bits._IncSize=/*1024*/16384; _Position=0;}
	~BitArray() {Bits.~ByteArray();}
	DWORD _Position;
	void Write(DWORD Number, BYTE SizeInBits);
	DWORD Read(BYTE SizeInBits);
	BYTE SingleRead();
	void SaveToFile(char* FileName);
	void LoadFromFile(char* FileName);
};
///////////////////////////////////////////////////////////
//BitArray::Write(Number, SizeInBits)
//  Number    SizeInBits 
///////////////////////////////////////////////////////////
void BitArray::Write(DWORD Number, BYTE SizeInBits)
{
	DWORD _byte, WPosition=_Position;
	BYTE CurrentWriting;

	__asm
   {
		mov ah, SizeInBits
      push ax
   	mov ebx, Number
      mov edx, WPosition
BA_W_Start:
      mov eax, edx
      and eax, 7
      neg eax
      add eax, 8         //al=CurrentRange
      mov edx, WPosition
      shr edx, 3
      mov _byte, edx
      pop dx
      mov ah, dh         //ah=CurrentSize
		cmp ah, al
      jb BA_W_End
      sub ah, al
		mov cl, ah
      mov edx, ebx
      shr edx, cl
      mov CurrentWriting, dl
      mov edx, 1
      shl edx, cl
      dec edx
      and ebx, edx
      push ax
      push ebx
	}
		Bits[_byte]+=CurrentWriting;
	__asm
   {
   	pop ebx
   	pop ax
   	push ax
		mov edx, WPosition
      mov ecx, 0
      mov cl, al
      add edx, ecx
		mov WPosition, edx
      jmp BA_W_Start
BA_W_End:
      cmp ah, 0
      je BA_W_End2
      sub al, ah
		mov cl, al
      mov edx, ebx
      shl edx, cl
      mov CurrentWriting, dl
		mov edx, WPosition
      mov ecx, 0
      mov cl, ah
      add edx, ecx
		mov WPosition, edx
	}
		Bits[_byte]+=CurrentWriting;
	__asm
   {
BA_W_End2:
   }
   _Position=WPosition;
}
///////////////////////////////////////////////////////////
//BitArray::Read(SizeInBits)
//     SizeInBits 
///////////////////////////////////////////////////////////
DWORD BitArray::Read(BYTE SizeInBits)
{
	DWORD _byte, RPosition=_Position, result;
	BYTE CurrentReading;

	__asm
   {
		mov ah, SizeInBits
      push ax
   	mov ebx, 0
      mov edx, RPosition
BA_R_Start:
      mov eax, edx
      and eax, 7
      neg eax
      add eax, 8             ;//al=CurrentRange
      mov edx, RPosition
      shr edx, 3
      mov _byte, edx
      pop dx
      mov ah, dh             ;//ah=CurrentSize
		cmp ah, al
      jb BA_R_End
      mov edx, 1
      mov cl, al
      shl edx, cl
      dec edx
      mov CurrentReading, dl
      push ax
      push ebx
	}
		CurrentReading&=Bits[_byte];
	__asm
   {
   	pop ebx
   	pop ax
      shl ebx, 8
      mov edx, 0
      mov dl, CurrentReading
      add ebx, edx
      sub ah, al
   	push ax
		mov edx, RPosition
      mov ecx, 0
      mov cl, al
      add edx, ecx
		mov RPosition, edx
      jmp BA_R_Start
BA_R_End:
      cmp ah, 0
      je BA_R_End2
      sub al, ah
		mov CurrentReading, al
      push ax
      push ebx
	}
		CurrentReading=Bits[_byte]>>CurrentReading;
	__asm
   {
   	pop ebx
   	pop ax
      mov cl, ah
      mov edx, 1
      shl edx, cl
      dec edx
      mov cl, ah
      shl ebx, cl
      mov ecx, 0
      mov cl, CurrentReading
      and ecx, edx
      add ebx, ecx
		mov edx, RPosition
      mov ecx, 0
      mov cl, ah
      add edx, ecx
		mov RPosition, edx
BA_R_End2:
      mov result, ebx
   }
  	_Position=RPosition;
	return result;
}
BYTE BitArray::SingleRead()
{
	DWORD _byte, RPosition=_Position;
	BYTE result;
	BYTE CurrentReading;

	__asm
   {
      mov ecx, RPosition
      and ecx, 7
      neg ecx
      add ecx, 7
      mov edx, RPosition
      shr edx, 3
      mov _byte, edx
      push ecx
	}
		CurrentReading=Bits.Data[_byte];
	__asm
   {
   	pop ecx
      mov dl, CurrentReading
		shr dl, cl
		and dl, 1
		mov result, dl
   }
  	_Position++;
	return result;
}
///////////////////////////////////////////////////////////
//BitArray::SaveToFile(FileName)
//    FileName
///////////////////////////////////////////////////////////
void BitArray::SaveToFile(char* FileName)
{
	DWORD SizeInBytes, nBytesWrite;

	HANDLE fout=CreateFile(FileName,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);
	SizeInBytes=((_Position-1)>>3)+1;
	WriteFile(fout, Bits.Data, SizeInBytes, &nBytesWrite, NULL);
	CloseHandle(fout);
}
///////////////////////////////////////////////////////////
//BitArray::LoadFromFile(FileName)
//    FileName
///////////////////////////////////////////////////////////
void BitArray::LoadFromFile(char* FileName)
{
	DWORD nBytesToRead, nBytesRead;

   _Position=0;
	HANDLE fin=CreateFile(FileName,GENERIC_READ,0,0,OPEN_EXISTING,0,0);
	nBytesToRead=GetFileSize(fin, 0);
	Bits[nBytesToRead+1]=0;
	ReadFile(fin, Bits.Data, nBytesToRead, &nBytesRead, NULL);
	Bits[nBytesToRead+1]=0;
	CloseHandle(fin);
}
///////////////////////////////////////////////////////////

#endif

