//	DynArray.h
//	Dynamic Array
//	version 3.0 [15.06.2000]
//	Copyright by SkyLord [Team 16]

#ifndef __DynArray_h
#define __DynArray_h

#include "Sorting.h"

#define MaxLong1 2147483647
////////////////////////////////////////////////////////////////////////////////
//===ByteArray==================================================================
//  BYTE':
//... DwordArray...
////////////////////////////////////////////////////////////////////////////////
class ByteArray
{
private:
	HGLOBAL block;
public:
	BYTE *Data;
	DWORD _IncSize;
	DWORD _Size;
	BYTE& operator [](DWORD Position)
	{
		ReSize(Position+1);
		return Data[Position];
	}
	void ReSize(DWORD NewSize);
	ByteArray()
	{
		_Size=0;
		_IncSize=256;
	}
	~ByteArray()
	{
		GlobalUnlock(block);
		GlobalFree(block);
	}
};
////////////////////////////////////////////////////////////////////////////////
void ByteArray::ReSize(DWORD NewSize)
{
   if(NewSize>_Size)
   {
		DWORD tmpSize=(((NewSize-1)/_IncSize)+1)*_IncSize;
		if(_Size==0)
		  	block=GlobalAlloc(GMEM_MOVEABLE, tmpSize);
      else
	   	block=GlobalReAlloc(block, tmpSize, GMEM_MOVEABLE);
      Data=(BYTE*)GlobalLock(block);
		for(DWORD tmp=_Size; tmp<tmpSize; tmp++) Data[tmp]=0;
		_Size=tmpSize;
   }
}
////////////////////////////////////////////////////////////////////////////////
//End of ByteArray
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//===ElementRef=================================================================
//-  
////////////////////////////////////////////////////////////////////////////////
struct ElementRef
{
	DWORD Element;
// 
	DWORD Frequency;
//    
	DWORD CodeLength;
//   
	DWORD Code;
//  
};
//////////////////////////////////////////////////////////////
ElementRef Blank={2147483647, 0, 0, 0};//  
//////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//===ElementArray===============================================================
//  ElementRef':
//... DwordArray...
////////////////////////////////////////////////////////////////////////////////
class ElementArray
{
private:
	HGLOBAL block;

	DWORD AddElement(DWORD Element, DWORD Position);
//      
//    -  Position
	void CodeLen(DWORD *A1, DWORD *A2, DWORD Position, DWORD Range, DWORD Code, BYTE Level);
//       
public:
	ElementRef *Data;
	DWORD _Position;
	DWORD _IncSize;
	DWORD _Size;
	DwordArray Links, LinksToCodes, ReLinks;

   ElementRef& operator [](DWORD Position)
	{
		return LoadData(Position);
	}
	ElementRef& LoadData(DWORD Position)
	{
		ReSize(Position+1);
		if(_Position<Position) _Position=Position;
		return Data[Position];
	}
	void ReSize(DWORD NewSize);

	long FindElement(DWORD Element);
//     
//    -   ,     
//    -  -X-1,  X -      ,  
//        
	DWORD AddElementIf(DWORD Element);
//     ,     
//    -   
	DWORD Size()
	{
		return _Position;
	};
//     
	DWORD Huffman(DWORD NumLastElement, BYTE IncludeOnce);
//
	BYTE CompareCodes(DWORD Code1, BYTE Length1, DWORD Position);

	ElementArray()
	{
		_Size=0;
		_IncSize=256;
		_Position=0;
	}
	~ElementArray()
	{
		GlobalUnlock(block);
		GlobalFree(block);
		Links.~DwordArray();
		ReLinks.~DwordArray();
		LinksToCodes.~DwordArray();
	}
};
////////////////////////////////////////////////////////////////////////////////
void ElementArray::ReSize(DWORD NewSize)
{
   if(NewSize>_Size)
   {
		DWORD tmpSize=(((NewSize-1)/_IncSize)+1)*_IncSize;
		if(_Size==0)
		  	block=GlobalAlloc(GMEM_MOVEABLE, tmpSize*sizeof(ElementRef));
      else
	   	block=GlobalReAlloc(block, tmpSize*sizeof(ElementRef), GMEM_MOVEABLE);
      Data=(ElementRef*)GlobalLock(block);
		for(DWORD tmp=_Size; tmp<tmpSize; tmp++) Data[tmp]=Blank;
		_Size=tmpSize;
   }
}
////////////////////////////////////////////////////////////////////////////////
long ElementArray::FindElement(DWORD Element)
{
	if(_Position==0) return -1;

	DWORD tmp, LeftRange=0, RightRange=_Position-1;
   ElementRef *pt=Data;
	__asm
   {
		mov esi, pt
// esi = pointer to Data
Find_While:
   	mov edx, LeftRange
      cmp edx, RightRange
      je Find_Eq
// if(LeftRange==RightRange) jump Find_Eq
		add edx, RightRange
      shr edx, 1
// edx = tmpMed = (LeftRange+RightRange)/2
		mov eax, edx
		shl eax, 4
      mov ecx, dword ptr [esi+eax]
// ecx = Data[tmpMed].Element
      cmp ecx, Element
      jae Find_AboveEq
// if(Data[tmpMed].Element>=Element) jump Find_AboveEq
      inc edx
      mov LeftRange, edx
// LeftRange = tmpMed+1
      jmp Find_While
Find_AboveEq:
      mov RightRange, edx
// RightRange = tmpMed
      jmp Find_While
Find_Eq:
		mov edx, RightRange
		mov eax, edx
		shl eax, 4
      mov ecx, dword ptr [esi+eax]
// ecx = Data[RightRange].Element
      cmp ecx, Element
      ja Find_Above
// if(Data[RightRange].Element>Element) jump Find_Above
      jb Find_Below
// if(Data[RightRange].Element<Element) jump Find_Below
      mov tmp, edx
// tmp=RightRange
	   jmp Find_End
Find_Above:
		mov eax, RightRange
      neg eax
      dec eax
      mov tmp, eax
// tmp=-RightRange-1
	   jmp Find_End
Find_Below:
		mov eax, RightRange
      neg eax
      sub eax, 2
// tmp=-RightRange-2
      mov tmp, eax
Find_End:
   }
   return tmp;
}
///////////////////////////////////////////////////////////
DWORD ElementArray::AddElement(DWORD Element, DWORD Position)
{
	if(_Position>0)
   {
		LoadData(_Position);
		for(DWORD tmp=_Position; tmp>Position; tmp--) LoadData(tmp)=LoadData(tmp-1);
   }
	LoadData(Position).Element=Element;
   _Position++;
	return Position;
}
///////////////////////////////////////////////////////////
DWORD ElementArray::AddElementIf(DWORD Element)
{
	long tmp=FindElement(Element);
	if(tmp>=0)
		return tmp;
	else
		return AddElement(Element, -tmp-1);
}
///////////////////////////////////////////////////////////
void ElementArray::CodeLen(DWORD *A1, DWORD *A2, DWORD Position, DWORD Range, DWORD Code, BYTE Level)
{
	if(A1[Position]>Range)
		CodeLen(A1, A2, A1[Position], Range, Code<<1, Level+1);
	else
		Data[Links.Data[A1[Position]]].CodeLength=Level;
	if(A2[Position]>Range)
		CodeLen(A1, A2, A2[Position], Range, (Code+1)<<1, Level+1);
	else
		Data[Links.Data[A2[Position]]].CodeLength=Level;
}
///////////////////////////////////////////////////////////
DWORD ElementArray::Huffman(DWORD NumLastElement, BYTE IncludeOnce)
{
	DWORD tmp1, tmp2;

	if(IncludeOnce==0)
	{
		DWORD *pt=(DWORD *)Data+1;
		__asm
		{
			mov eax, pt
			mov ebx, NumLastElement
			inc ebx
			shl ebx, 4
Huffman_Cycle1:
			sub ebx, 16
			cmp dword ptr [eax+ebx], 2
			jae Huffman_Cycle1_Med
			mov dword ptr [eax+ebx], 0
Huffman_Cycle1_Med:
			cmp ebx, 0
			jne Huffman_Cycle1
		}
	}
	Links[NumLastElement]=0;
	_GlobalSortParam.ElementsPointer=(DWORD*)Data;
	_GlobalSortParam.LinksPointer=Links.Data;
	_GlobalSortParam.StructSize=sizeof(ElementRef);
	_GlobalSortParam.StructOffset=4;
	_GlobalSortParam.Vector=1;
	Sorting(0, NumLastElement, 0);
	DWORD *pt1=(DWORD*)Data+1;
	DWORD *pt2=Links.Data;
	DWORD NLElement=_Position;
	__asm
	{
		mov edi, pt1
		mov esi, pt2
		mov ebx, 0
Huffman_Cycle2:
		mov eax, dword ptr [esi+4*ebx]
		shl eax, 4
		cmp dword ptr [edi+eax], 0
		je Huffman_Cycle2_End
		cmp ebx, NumLastElement
		inc ebx
		jb Huffman_Cycle2
Huffman_Cycle2_End:
		dec ebx
		mov tmp2, ebx
	}

	DwordArray tArray1;
	DwordArray tArray2;
	DwordArray tArray3;
	tArray1[tmp2*2+1]=0;
	tArray2[tmp2*2+1]=0;
	tArray3[tmp2*2+1]=0;
   for(tmp1=0; tmp1<=tmp2; tmp1++) tArray1.Data[tmp1]=Data[Links.Data[tmp1]].Frequency;
	pt1=tArray1.Data;
	pt2=tArray2.Data;
	DWORD *pt3=tArray3.Data;
	DWORD lmin1, lmin2;
	DWORD pmin1, pmin2;
	DWORD EndPosition=tmp2<<1;
	__asm
	{
		mov esi, pt1
		mov eax, tmp2
      cmp tmp2, 0
      je Huff_Not
Huff_Start1:
		inc eax
		mov ebx, eax
		mov lmin1, MaxLong1
		mov lmin2, MaxLong1
Huff_Start2:
		cmp ebx, 0
		je Huff_Min_End
		dec ebx
		mov edx, dword ptr [esi+4*ebx]
		cmp edx, lmin1
		jb Huff_Start3
		cmp edx, lmin2
		jae Huff_Start2
		mov lmin2, edx
		mov pmin2, ebx
		jmp Huff_Start2
Huff_Start3:
		push lmin1
      pop lmin2
		push pmin1
      pop pmin2
		mov lmin1, edx
		mov pmin1, ebx
		jmp Huff_Start2
Huff_Min_End:
		mov ebx, pmin1
		mov dword ptr [esi+4*ebx], MaxLong1
		mov ebx, pmin2
		mov dword ptr [esi+4*ebx], MaxLong1
		mov ebx, lmin1
		add ebx, lmin2
		mov dword ptr [esi+4*eax], ebx
		mov edi, pt2
		mov ecx, pmin1
		mov dword ptr [edi+4*eax], ecx
		mov edi, pt3
		mov ecx, pmin2
		mov dword ptr [edi+4*eax], ecx
		cmp eax, EndPosition
		jb Huff_Start1
Huff_Not:
	}
   if(tmp2!=0)
		CodeLen(tArray2.Data, tArray3.Data, tmp2*2, tmp2, 0, 1);
   else
   {
   	Data[Links.Data[0]].CodeLength=1;
   	Data[Links.Data[0]].Code=0;
   }
   LinksToCodes[NumLastElement]=0;
	_GlobalSortParam.ElementsPointer=(DWORD *)Data;
	_GlobalSortParam.LinksPointer=LinksToCodes.Data;
	_GlobalSortParam.StructSize=sizeof(ElementRef);
	_GlobalSortParam.StructOffset=8;
	_GlobalSortParam.Vector=0;
	Sorting(0, NumLastElement, 0);
	pt1=(DWORD*)Data+2;
	pt2=LinksToCodes.Data;
	__asm
	{
		mov esi, pt2
		mov edi, pt1
		mov eax, dword ptr [esi]
		shl eax, 4
		cmp dword ptr [edi+eax], 0
		jne CodeSyntesing_Start
		mov dword ptr [edi+eax], 1
CodeSyntesing_Start:
		mov dword ptr [edi+eax+4], 0
		mov ebx, 1
		mov eax, 0
CodeSyntesing_Cycle:
		inc eax
		mov edx, dword ptr [esi+4*ebx]
		shl edx, 4
		mov ecx, dword ptr [edi+edx]
		mov edx, dword ptr [esi+4*ebx-4]
		shl edx, 4
		sub ecx, dword ptr [edi+edx]
		shl eax, cl
		mov edx, dword ptr [esi+4*ebx]
		shl edx, 4
		mov dword ptr [edi+edx+4], eax
		inc ebx
		cmp ebx, NumLastElement
		jbe CodeSyntesing_Cycle
	}
   ReLinks[NumLastElement]=0;
   for(tmp1=0; tmp1<=NumLastElement; tmp1++) ReLinks.Data[tmp1]=16777215;
   for(tmp1=0; tmp1<=tmp2; tmp1++) ReLinks.Data[Links.Data[tmp1]]=tmp1;
   return tmp2;
}
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////
// :
// 0 -    
// 1 -    
// 2 -    
// 3 -  
///////////////////////////////////////////////////////////
BYTE ElementArray::CompareCodes(DWORD Code1, BYTE Length1, DWORD Position)
{
   BYTE result;
	DWORD Code2=Data[LinksToCodes.Data[Position]].Code;
	BYTE Length2=Data[LinksToCodes.Data[Position]].CodeLength;
   __asm
   {
   	mov dl, Length1
   	mov dh, Length2
   	cmp dl, dh
      ja CC_Above1
      mov cl, dh
      sub cl, dl
		mov eax, Code1
		mov ebx, Code2
      shr ebx, cl
      jmp CC_Cont
CC_Above1:
      mov cl, dl
      sub cl, dh
		mov eax, Code1
		mov ebx, Code2
      shr eax, cl
CC_Cont:
      cmp eax, ebx
		ja CC_Above2
		jb CC_Below2
   	cmp dl, dh
      jne CC_NE
      mov result, 3
      jmp CC_End
CC_NE:
		mov result, 1
      jmp CC_End
CC_Above2:
		mov result, 2
      jmp CC_End
CC_Below2:
		mov result, 0
CC_End:
   }
   return result;
}
///////////////////////////////////////////////////////////


#endif
