//	RCImage.h
//	Image object for RC1
//	version 2.7 [15.06.2000]
//	Copyright by SkyLord [Team 16]

#ifndef __RCImage_h
#define __RCImage_h

#define MaxLong1 2147483647
#define MaxLong2 2147483646
#define MaxLong3 2147483645

#define BYTE unsigned char
#define WORD unsigned int
#define DWORD unsigned long

#define CurrentMajorVersion 1
#define CurrentMinorVersion 0 //1

#include "DynArray.h"
#include "BitArray.h"

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
class RCImage
{
public:
	WORD MaxXCoord, MaxYCoord;
	DWORD Progress, MaxProgress;
	BYTE Error;
	BYTE Tolerance, QBluring;
	BYTE Diff[4], VDiff[4], BP[4], NTone[4];
	DWORD NumOfDiffColors, NumOfFileRecords;
	DWORD CurrentCode;
	DWORD ColorIsWidthUnite, ColorIsHeightUnite, ColorIsRGBASwitch;
	DWORD SysFreq[3], Sys[3];
	DWORD TonesFreq[4][256];
	BYTE ChangedTones[4][256];
	BYTE Tones[4][256];
	DwordArray ImageInArray;
	DwordArray Used;
	DWORD **Image;
	DWORD **ImageB;
//	DwordArray *Image;
//	DwordArray *ImageB;
	DWORD Posing;
	BitArray File;
	ElementArray Palette;

	void ReSizeImage();
	void CloseImage();

	DWORD ReadNextCode();

	void RecodeColor(DWORD Color, BYTE* A);
	DWORD CodeColor(BYTE* A);

	void Blur(WORD x, WORD y, BYTE Width, BYTE Height);
	DWORD ToleranceTest(WORD x, WORD y, BYTE Width, BYTE Height);
	void RecurceLoad(WORD x, WORD y, BYTE Size, BYTE DoBlur);
	void RecurceSave(WORD x, WORD y, BYTE size);

	void PaintToImage();
	DWORD WriteToArray();

	void GetSysColors();
	void DelTone(BYTE DeletingTone);
	void RepaintColors();

	void LoadImage(char* FileName);
	void SaveImage(char* FileName);
	void SaveAsBitmap(char* FileName);
	void SimulateSave();
	void (*ProgressFunctor)(DWORD, DWORD);
	void ShowProgress() {if(ProgressFunctor!=NULL) ProgressFunctor(Progress, MaxProgress);}

   RCImage(char *FileName, void(*PF)(DWORD, DWORD))
   {
		ProgressFunctor=PF;
		LoadImage(FileName);
   }
   RCImage(WORD x, WORD y, void(*PF)(DWORD, DWORD))
   {
		ProgressFunctor=PF;
   	MaxXCoord=x;
      MaxYCoord=y;
		Error=0;
      ReSizeImage();
   }
   ~RCImage()
   {
		Palette.~ElementArray();
		ImageInArray.~DwordArray();
		File.~BitArray();
      Used.~DwordArray();
      CloseImage();
   }
};
///////////////////////////////////////////////////////////
void RCImage::ReSizeImage()
{
	WORD tmpx;
	Image=new DWORD*[MaxXCoord+1];
	for(tmpx=0; tmpx<=MaxXCoord; tmpx++) Image[tmpx]=new DWORD[MaxYCoord+1];
	ImageB=new DWORD*[MaxXCoord+1];
	for(tmpx=0; tmpx<=MaxXCoord; tmpx++) ImageB[tmpx]=new DWORD[MaxYCoord+1];
/*	WORD tmpx;
	Image=new DwordArray[MaxXCoord+1];
	ImageB=new DwordArray[MaxXCoord+1];*/
}
///////////////////////////////////////////////////////////
void RCImage::CloseImage()
{
	WORD tmpx;
	for(tmpx=0; tmpx<=MaxXCoord; tmpx++) delete[] Image[tmpx];
	delete[] Image;
  	for(tmpx=0; tmpx<=MaxXCoord; tmpx++) delete[] ImageB[tmpx];
	delete[] ImageB;
/*	WORD tmpx;
	for(tmpx=0; tmpx<=MaxXCoord; tmpx++) Image[tmpx].~DwordArray();
	delete[] Image;
  	for(tmpx=0; tmpx<=MaxXCoord; tmpx++) ImageB[tmpx].~DwordArray();
	delete[] ImageB;*/
}
///////////////////////////////////////////////////////////
DWORD RCImage::ReadNextCode()
{
	DWORD StartPos, StopPos, tmpMed;
	DWORD tCode=0;
   BYTE tmp;
   BYTE Len=1;

//	if(File.Read(1)==1) tCode++;
	if(File.SingleRead()==1) tCode++;
	StartPos=0;
	StopPos=NumOfDiffColors;
   __asm
   {
   	mov ebx, StartPos
   	mov edx, StopPos
RNC_While:
		mov ecx, ebx
      add ecx, edx
      shr ecx, 1
      mov tmpMed, ecx
      push ebx
      push edx
	}
		tmp=Palette.CompareCodes(tCode, Len, tmpMed);
   __asm
   {
   	pop edx
      pop ebx
		mov ecx, ebx
      inc ecx
      cmp ecx, edx
      je RNC_Eq
		cmp tmp, 2
      je RNC_T2
      jb RNC_T0_1
      ja RNC_End
RNC_T0_1:
		cmp tmp, 1
      je RNC_T1
      mov edx, tmpMed
      jmp RNC_While
RNC_T1:
      push ebx
      push edx
	}
		tCode<<=1;
		Len++;
//		if(File.Read(1)==1) tCode++;
		if(File.SingleRead()==1) tCode++;
	__asm
   {
   	pop edx
      pop ebx
      jmp RNC_While
RNC_T2:
      mov ebx, tmpMed
      jmp RNC_While
RNC_Eq:
		cmp tmp, 3
      je RNC_End
		mov tmp, 3
      mov ecx, tmpMed
      inc ecx
      mov tmpMed, ecx
RNC_End:
		cmp tmp, 3
   	jne RNC_While
   }
   return Palette.LinksToCodes.Data[tmpMed];
}
///////////////////////////////////////////////////////////
void RCImage::RecodeColor(DWORD Color, BYTE* A)
{
   __asm
   {
      mov eax, Color;
   	mov esi, A;
      mov [esi], eax;
   }
}
///////////////////////////////////////////////////////////
DWORD RCImage::CodeColor(BYTE* A)
{
	DWORD result;
   __asm
   {
   	mov esi, A
      mov eax, [esi]
      mov result, eax
   }
   return result;
}
///////////////////////////////////////////////////////////
DWORD abs(DWORD x)
{
	DWORD result;
   __asm
   {
   	mov eax, x
      cmp eax, 0
      jge abs_1
      neg eax
abs_1:
      mov result, eax
   }
   return result;
}
///////////////////////////////////////////////////////////
void RCImage::Blur(WORD x, WORD y, BYTE Width, BYTE Height)
{
	DWORD tmp1, tmp2, tmp3, XSize=1<<Width, YSize=1<<Height;
	DWORD k0, k1;
	DWORD CurrentColor;
	BYTE tmpc1[4], tmpc3[4], tmpc4[4], tmpL[4], tmpR[4], tmpU[4], tmpD[4];
   DWORD temp[4];

	CurrentColor=Image[x][y];
	RecodeColor(CurrentColor, tmpc1);
	for(tmp1=x; tmp1<x+XSize; tmp1++)
		for(tmp2=y; tmp2<y+YSize; tmp2++)
			if(Width>0 && Height>0)
			{
				RecodeColor(CurrentColor, tmpL);
				if(x>0)
				{
					RecodeColor(Image[x-1][tmp2], tmpc3);
					for(tmp3=0; tmp3<4; tmp3++)
					{
						if(abs(tmpc3[tmp3]-tmpc1[tmp3])<=QBluring) tmpL[tmp3]=tmpc3[tmp3];
                  temp[tmp3]=(tmpL[tmp3]+tmpc1[tmp3])>>1;
//						tmpL[tmp3]=(tmpL[tmp3]+tmpc1[tmp3])>>1;
						tmpL[tmp3]=temp[tmp3];
					}
				}
				else RecodeColor(CurrentColor, tmpL);

				RecodeColor(CurrentColor, tmpR);
				if(x+XSize-1<MaxXCoord)
				{
					RecodeColor(Image[x+XSize][tmp2], tmpc3);
					for(tmp3=0; tmp3<4; tmp3++)
					{
						if(abs(tmpc3[tmp3]-tmpc1[tmp3])<=QBluring) tmpR[tmp3]=tmpc3[tmp3];
						temp[tmp3]=(tmpR[tmp3]+tmpc1[tmp3])>>1;
//						tmpR[tmp3]=(tmpR[tmp3]+tmpc1[tmp3])>>1;
						tmpR[tmp3]=temp[tmp3];
					}
				}
				else RecodeColor(CurrentColor, tmpR);

				RecodeColor(CurrentColor, tmpU);
				if(y>0)
				{
					RecodeColor(Image[tmp1][y-1], tmpc3);
					for(tmp3=0; tmp3<4; tmp3++)
					{
						if(abs(tmpc3[tmp3]-tmpc1[tmp3])<=QBluring) tmpU[tmp3]=tmpc3[tmp3];
						temp[tmp3]=(tmpU[tmp3]+tmpc1[tmp3])>>1;
//						tmpU[tmp3]=(tmpU[tmp3]+tmpc1[tmp3])>>1;
						tmpU[tmp3]=temp[tmp3];
					}
				}
				else RecodeColor(CurrentColor, tmpU);

				RecodeColor(CurrentColor, tmpD);
				if(y+YSize-1<MaxYCoord)
				{
					RecodeColor(Image[tmp1][y+YSize], tmpc3);
					for(tmp3=0; tmp3<4; tmp3++)
					{
						if(abs(tmpc3[tmp3]-tmpc1[tmp3])<=QBluring) tmpD[tmp3]=tmpc3[tmp3];
						temp[tmp3]=(tmpD[tmp3]+tmpc1[tmp3])>>1;
//						tmpD[tmp3]=(tmpD[tmp3]+tmpc1[tmp3])>>1;
						tmpD[tmp3]=temp[tmp3];
					}
				}
				else RecodeColor(CurrentColor, tmpD);

				k0=x+XSize-1-tmp1;
				k1=tmp1-x;
				for(tmp3=0; tmp3<4; tmp3++)
            {
					temp[tmp3]=(tmpL[tmp3]*k0+tmpR[tmp3]*k1)/(XSize-1);
//					tmpc3[tmp3]=int((tmpL[tmp3]*k0+tmpR[tmp3]*k1)/(XSize-1));
					if(temp[tmp3]<=255)
						tmpc3[tmp3]=temp[tmp3];
               else
						tmpc3[tmp3]=255;
            }
				k0=y+YSize-1-tmp2;
				k1=tmp2-y;
				for(tmp3=0; tmp3<4; tmp3++)
            {
					temp[tmp3]=(tmpU[tmp3]*k0+tmpD[tmp3]*k1)/(YSize-1);
//					tmpc4[tmp3]=int((tmpU[tmp3]*k0+tmpD[tmp3]*k1)/(YSize-1));
					if(temp[tmp3]<=255)
						tmpc4[tmp3]=temp[tmp3];
               else
						tmpc4[tmp3]=255;
            }
				for(tmp3=0; tmp3<4; tmp3++)
            {
					temp[tmp3]=(tmpc3[tmp3]+tmpc4[tmp3])>>1;
//					tmpc3[tmp3]=(tmpc3[tmp3]+tmpc4[tmp3])>>1;
					if(temp[tmp3]<=255)
						tmpc3[tmp3]=temp[tmp3];
               else
						tmpc3[tmp3]=255;
            }
				Image[tmp1][tmp2]=CodeColor(tmpc3);
			}
}
///////////////////////////////////////////////////////////
DWORD Minimum(DWORD x, DWORD y)
{
	DWORD result;
   __asm
   {
   	mov eax, x
      mov ebx, y
		cmp eax, ebx
      ja Min_Above
		mov result, eax
      jmp Min_End
Min_Above:
		mov result, ebx
Min_End:
   }
	return result;
}
///////////////////////////////////////////////////////////
DWORD Maximum(DWORD x, DWORD y)
{
	DWORD result;
   __asm
   {
   	mov eax, x
      mov ebx, y
		cmp eax, ebx
      ja Max_Above
		mov result, ebx
      jmp Max_End
Max_Above:
		mov result, eax
Max_End:
   }
	return result;
}
///////////////////////////////////////////////////////////
DWORD RCImage::ToleranceTest(WORD x, WORD y, BYTE Width, BYTE Height)
{
	DWORD tmp1, tmp2, tmp3, XSize=1<<Width, YSize=1<<Height;
	BYTE MinParam[4], MaxParam[4], CurParam[4];
	DWORD Med[4];
	BOOL test;

	if(x+XSize>MaxXCoord+1) return MaxLong1;
	if(y+YSize>MaxYCoord+1) return MaxLong1;
	test=true;
	for(tmp1=0; tmp1<4; tmp1++) MinParam[tmp1]=255;
	for(tmp1=0; tmp1<4; tmp1++) MaxParam[tmp1]=0;
	for(tmp1=0; tmp1<4; tmp1++) Med[tmp1]=0;
	tmp1=x;
	tmp2=y;
	while(test==true && tmp1<x+XSize)
	{
		RecodeColor(Image[tmp1][tmp2], CurParam);
		for(tmp3=0; tmp3<4; tmp3++)
			if(CurParam[tmp3]<MinParam[tmp3]) MinParam[tmp3]=CurParam[tmp3];
		for(tmp3=0; tmp3<4; tmp3++)
			if(CurParam[tmp3]>MaxParam[tmp3]) MaxParam[tmp3]=CurParam[tmp3];
		for(tmp3=0; tmp3<4; tmp3++)
			if(MaxParam[tmp3]-MinParam[tmp3]>Tolerance) test=false;
		for(tmp3=0; tmp3<4; tmp3++)
			Med[tmp3]+=CurParam[tmp3];
		tmp2++;
		if(tmp2==y+YSize)
		{
			tmp1++;
			tmp2=y;
		}
	}
	if(test==false)
		return MaxLong1;
	else
	{
		for(tmp3=0; tmp3<4; tmp3++) Med[tmp3]>>=Width+Height;
		for(tmp3=0; tmp3<4; tmp3++) CurParam[tmp3]=Med[tmp3];
		return CodeColor(CurParam);
	}
}
///////////////////////////////////////////////////////////
void RCImage::RecurceLoad(WORD x, WORD y, BYTE Size, BYTE DoBlur)
{
// DoBlur=0 FirstLoad
// DoBlur=1 SecondLoad
	DWORD tmp1, tmp2, tmp3, tmp4, XSize, YSize;

	if(x<=MaxXCoord && y<=MaxYCoord)
		if(Size>0)
		{
			RecurceLoad(x, y, Size-1, DoBlur);
			RecurceLoad(x, y+(1<<(Size-1)), Size-1, DoBlur);
			RecurceLoad(x+(1<<(Size-1)), y, Size-1, DoBlur);
			RecurceLoad(x+(1<<(Size-1)), y+(1<<(Size-1)), Size-1, DoBlur);
		}
		else
		{
			if((DoBlur==0 && Image[x][y]==MaxLong1) || (DoBlur==1 && ImageB[x][y]==MaxLong1))
			{
				for(tmp1=0; ImageInArray[Posing]==ColorIsWidthUnite; Posing++) tmp1++;
				for(tmp2=0; ImageInArray[Posing]==ColorIsHeightUnite; Posing++) tmp2++;
				XSize=1<<tmp1;
				YSize=1<<tmp2;
				if(DoBlur==0)
				{
					for(tmp3=x; tmp3<x+XSize; tmp3++)
						for(tmp4=y; tmp4<y+YSize; tmp4++)
							Image[tmp3][tmp4]=ImageInArray[Posing];
				}
				else
				{
					for(tmp3=x; tmp3<x+XSize; tmp3++)
						for(tmp4=y; tmp4<y+YSize; tmp4++)
							ImageB[tmp3][tmp4]=ImageInArray[Posing];
					Blur(x, y, tmp1, tmp2);
				}
				Posing++;
				Progress+=XSize*YSize;
				ShowProgress();
			}
		}
}
///////////////////////////////////////////////////////////
void RCImage::RecurceSave(WORD x, WORD y, BYTE Size)
{
	DWORD tmp1, tmp2, XSize, YSize;
   DWORD Color;

	if(x<=MaxXCoord && y<=MaxYCoord)
		if(Size>0)
		{
			RecurceSave(x, y, Size-1);
			RecurceSave(x, y+(1<<(Size-1)), Size-1);
			RecurceSave(x+(1<<(Size-1)), y, Size-1);
			RecurceSave(x+(1<<(Size-1)), y+(1<<(Size-1)), Size-1);
		}
		else
			if(Image[x][y]!=MaxLong1)
			{
				XSize=0;
				YSize=0;
				for(tmp1=0; ToleranceTest(x, y, tmp1, 0)!=MaxLong1; tmp1++)
				{
					for(tmp2=0; ToleranceTest(x, y, tmp1, tmp2)!=MaxLong1; tmp2++) ;
					tmp2--;
					if(tmp1+tmp2>XSize+YSize)
					{
						XSize=tmp1;
						YSize=tmp2;
					}
				}
				Color=ToleranceTest(x, y, XSize, YSize);
				for(tmp1=0; tmp1<XSize; tmp1++) ImageInArray[Posing+tmp1]=MaxLong1;
				Posing+=XSize;
				SysFreq[0]+=XSize;
				for(tmp1=0; tmp1<YSize; tmp1++) ImageInArray[Posing+tmp1]=MaxLong2;
				Posing+=YSize;
				SysFreq[1]+=YSize;
				ImageInArray[Posing]=Color;
				Posing++;

				Palette[Palette.AddElementIf(Color)].Frequency++;

				XSize=1<<XSize;
				YSize=1<<YSize;
				for(tmp1=x; tmp1<x+XSize; tmp1++)
					for(tmp2=y; tmp2<y+YSize; tmp2++)
						Image[tmp1][tmp2]=MaxLong1;
				Progress+=XSize*YSize;
				ShowProgress();
			}
}
///////////////////////////////////////////////////////////
void RCImage::PaintToImage()
{
	DWORD tmp1, tmp2, Quadrix;

	if(MaxXCoord>MaxYCoord)
		Quadrix=BitsPerNumber(MaxXCoord);
	else
		Quadrix=BitsPerNumber(MaxYCoord);

	for(tmp1=0; tmp1<=MaxXCoord; tmp1++)
		for(tmp2=0; tmp2<=MaxYCoord; tmp2++)
			Image[tmp1][tmp2]=MaxLong1;
	Posing=0;
	RecurceLoad(0, 0, Quadrix, 0);
	for(tmp1=0; tmp1<=MaxXCoord; tmp1++)
		for(tmp2=0; tmp2<=MaxYCoord; tmp2++)
			ImageB[tmp1][tmp2]=MaxLong1;
	Posing=0;
	RecurceLoad(0, 0, Quadrix, 1);
}
///////////////////////////////////////////////////////////
DWORD RCImage::WriteToArray()
{
	DWORD tmp1, tmp2, Quadrix;

	SysFreq[0]=0;
	SysFreq[1]=0;
	SysFreq[2]=0;
	for(tmp1=0; tmp1<=255; tmp1++)
		for(tmp2=0; tmp2<4; tmp2++)
			TonesFreq[tmp2][tmp1]=0;
	for(tmp1=0; tmp1<=255; tmp1++)
		for(tmp2=0; tmp2<4; tmp2++)
			Tones[tmp2][tmp1]=tmp1;
	RepaintColors();

	Posing=0;
	if(MaxXCoord>MaxYCoord)
		Quadrix=BitsPerNumber(MaxXCoord);
	else
		Quadrix=BitsPerNumber(MaxYCoord);
	RecurceSave(0, 0, Quadrix);
	return Posing-1;
}
///////////////////////////////////////////////////////////
void RCImage::GetSysColors()
{
	DWORD tmp1;
	long tmp2;

	for(tmp1=0; tmp1<Palette.Size(); tmp1++)
		if(Palette[tmp1].Frequency==1) SysFreq[2]++;
	for(tmp1=0; tmp1<3; tmp1++)
		if(SysFreq[tmp1]<2) SysFreq[tmp1]=2;
	for(tmp1=0; tmp1<3; tmp1++) Sys[tmp1]=MaxLong1;
	tmp2=0;
	for(tmp1=0; tmp1<3; tmp1++)
	{
		while(Palette.FindElement(tmp2)>=0) tmp2++;
		Sys[tmp1]=tmp2;
		tmp2=Palette.AddElementIf(Sys[tmp1]);
		Palette[tmp2].Frequency=SysFreq[tmp1];
	}
}
///////////////////////////////////////////////////////////
void RCImage::DelTone(BYTE DTone)
{
	DWORD tmp1, tmp2, tmp4, tmp5, delta;
	DWORD CurX, CurY, CurM;

	CurX=MaxLong1;
	CurY=MaxLong1;
	CurM=MaxLong1;
	delta=MaxLong1;
	for(tmp1=0; tmp1<=254; tmp1++)
		if(TonesFreq[DTone][tmp1]>0)
			for(tmp2=tmp1+1; tmp2<=255; tmp2++)
				if(TonesFreq[DTone][tmp2]>0)
					for(tmp4=tmp1; tmp4<=tmp2; tmp4++)
					{
						tmp5=TonesFreq[DTone][tmp1]*(tmp4-tmp1)+TonesFreq[DTone][tmp2]*(tmp2-tmp4);
						if(tmp5<delta)
						{
							delta=tmp5;
							CurX=tmp1;
							CurY=tmp2;
							CurM=tmp4;
						}
					}
	for(tmp1=0; tmp1<=255; tmp1++)
		if(Tones[DTone][tmp1]==CurX || Tones[DTone][tmp1]==CurY) Tones[DTone][tmp1]=CurM;
	tmp1=TonesFreq[DTone][CurX];
	tmp2=TonesFreq[DTone][CurY];
	tmp4=TonesFreq[DTone][CurM];
	TonesFreq[DTone][CurX]=0;
	TonesFreq[DTone][CurY]=0;
	TonesFreq[DTone][CurM]=tmp1+tmp2+tmp4;
}
///////////////////////////////////////////////////////////
void RCImage::RepaintColors()
{
	DWORD tmp4, tmp1, tmp2, tmp3;
	BYTE CurParam[4];

	for(tmp1=0; tmp1<=MaxXCoord; tmp1++)
		for(tmp2=0; tmp2<=MaxYCoord; tmp2++)
		{
			RecodeColor(Image[tmp1][tmp2], CurParam);
			for(tmp3=0; tmp3<4; tmp3++) TonesFreq[tmp3][CurParam[tmp3]]++;
		}
	for(tmp4=0; tmp4<4; tmp4++)
	{
		Diff[tmp4]=0;
		for(tmp3=0; tmp3<=255; tmp3++)
			if(TonesFreq[tmp4][tmp3]>0) Diff[tmp4]++;
		for(Diff[tmp4]=Diff[tmp4]; Diff[tmp4]>VDiff[tmp4]+1; Diff[tmp4]--) DelTone(tmp4);
		tmp2=0;
		for(tmp1=0; TonesFreq[tmp4][tmp1]==0; tmp1++) ;
    	ChangedTones[tmp4][0]=Tones[tmp4][tmp1];
		for(tmp1=0; tmp1<=255; tmp1++)
		{
			if(TonesFreq[tmp4][tmp1]>0)
				if(Tones[tmp4][tmp1]!=ChangedTones[tmp4][tmp2])
				{
					tmp2++;
					ChangedTones[tmp4][tmp2]=Tones[tmp4][tmp1];
				}
		}
		for(tmp3=0; tmp3<=255; tmp3++)
			for(tmp1=0; tmp1<=tmp2; tmp1++)
				if(Tones[tmp4][tmp3]==ChangedTones[tmp4][tmp1]) Tones[tmp4][tmp3]=tmp1;
		Diff[tmp4]=tmp2+1;
	}

	for(tmp1=0; tmp1<=MaxXCoord; tmp1++)
		for(tmp2=0; tmp2<=MaxYCoord; tmp2++)
		{
			RecodeColor(Image[tmp1][tmp2], CurParam);
			for(tmp3=0; tmp3<4; tmp3++) CurParam[tmp3]=ChangedTones[tmp3][Tones[tmp3][CurParam[tmp3]]];
			Image[tmp1][tmp2]=CodeColor(CurParam);
		}
}
///////////////////////////////////////////////////////////
void RCImage::LoadImage(char* FileName)
{
	unsigned char rFLetter, rSLetter;
   BYTE rMajorVersion, rMinorVersion;
	DWORD tmp1, tmp2;

	Progress=0;
	MaxProgress=2*(MaxXCoord+1)*(MaxYCoord+1);

	File.LoadFromFile(FileName);
	rFLetter=File.Read(8);
	rSLetter=File.Read(8);
	rMajorVersion=File.Read(8);
	rMinorVersion=File.Read(8);
//----RC-header----
	MaxXCoord=File.Read(16);
	MaxYCoord=File.Read(16);
   ReSizeImage();
	Tolerance=File.Read(8);
	QBluring=File.Read(8);
	for(tmp1=0; tmp1<4; tmp1++) Diff[tmp1]=File.Read(8);
	for(tmp1=0; tmp1<4; tmp1++) BP[tmp1]=BitsPerNumber(Diff[tmp1]);
   BP[3]=0;
	NumOfDiffColors=File.Read(24);
	NumOfFileRecords=File.Read(24);
	Progress=0;
	MaxProgress=NumOfFileRecords;
	for(tmp1=0; tmp1<=NumOfDiffColors+1; tmp1++) Palette[tmp1].Element=MaxLong1;
/*	ColorIsWidthUnite=File.Read(24);
	Palette[File.Read(20)].Element=ColorIsWidthUnite;
	ColorIsHeightUnite=File.Read(24);
	Palette[File.Read(20)].Element=ColorIsHeightUnite;
	ColorIsRGBASwitch=File.Read(24);
	Palette[File.Read(20)].Element=ColorIsRGBASwitch;*/
	ColorIsWidthUnite=File.Read(24);
   tmp1=File.Read(20);
	Palette[tmp1].Element=ColorIsWidthUnite;
	ColorIsHeightUnite=File.Read(24);
   tmp1=File.Read(20);
	Palette[tmp1].Element=ColorIsHeightUnite;
	ColorIsRGBASwitch=File.Read(24);
   tmp1=File.Read(20);
	Palette[tmp1].Element=ColorIsRGBASwitch;
	for(tmp2=0; tmp2<4; tmp2++)
		for(tmp1=0; tmp1<=Diff[tmp2]; tmp1++)
			Tones[tmp2][tmp1]=File.Read(8);
//----CodeSizes----
	for(tmp1=0; tmp1<=NumOfDiffColors; tmp1++) Palette.Links[tmp1]=tmp1;
	for(tmp1=0; tmp1<=NumOfDiffColors; tmp1++) Palette[tmp1].CodeLength=0;
	for(tmp1=0; tmp1<=NumOfDiffColors; tmp1++)
   	Palette[tmp1].Frequency=File.Read(20);
	Palette.Huffman(NumOfDiffColors, 1);
//----CodeSizes----
	for(tmp1=0; tmp1<=NumOfFileRecords+2; tmp1++) ImageInArray[tmp1]=MaxLong1;
	for(tmp1=0; tmp1<=NumOfFileRecords; tmp1++)
	{
		CurrentCode=ReadNextCode();
		if(Palette[CurrentCode].Element==MaxLong1)
		{
			for(tmp2=0; tmp2<4; tmp2++) NTone[tmp2]=File.Read(BP[tmp2]);
			for(tmp2=0; tmp2<4; tmp2++) NTone[tmp2]=Tones[tmp2][NTone[tmp2]];
			Palette[CurrentCode].Element=CodeColor(NTone);
		}
		ImageInArray[tmp1]=Palette[CurrentCode].Element;
		if(ImageInArray[tmp1]==ColorIsRGBASwitch)
		{
			for(tmp2=0; tmp2<4; tmp2++) NTone[tmp2]=File.Read(BP[tmp2]);
			for(tmp2=0; tmp2<4; tmp2++) NTone[tmp2]=Tones[tmp2][NTone[tmp2]];
			ImageInArray[tmp1]=CodeColor(NTone);
		}
	}
	PaintToImage();
}
///////////////////////////////////////////////////////////
void RCImage::SaveImage(char* FileName)
{
	DWORD tmp1, tmp2, tmp3, tmp4;
	BYTE tmp[4];

	Progress=0;
	MaxProgress=3*(MaxXCoord+1)*(MaxYCoord+1);

	NumOfFileRecords=WriteToArray();
	ColorIsWidthUnite=MaxLong1;
	ColorIsHeightUnite=MaxLong2;
	PaintToImage();
//---A BIG, BIG, BIG OPTIMIZATION------
	GetSysColors();
	ColorIsWidthUnite=Sys[0];
	ColorIsHeightUnite=Sys[1];
	ColorIsRGBASwitch=Sys[2];
	NumOfDiffColors=Palette.Huffman(Palette.Size()-1, 1);
//-------------------------------------
//----RC-header----
	File.Write(82, 8);
	File.Write(67, 8);
	File.Write(CurrentMajorVersion, 8);
	File.Write(CurrentMinorVersion, 8);
//----RC-header----
	File.Write(MaxXCoord, 16);
	File.Write(MaxYCoord, 16);
	File.Write(Tolerance, 8);
	File.Write(QBluring, 8);
	for(tmp1=0; tmp1<4; tmp1++) File.Write(Diff[tmp1]-1, 8);
	for(tmp1=0; tmp1<4; tmp1++) BP[tmp1]=BitsPerNumber(Diff[tmp1]-1);
   BP[3]=0;
	File.Write(NumOfDiffColors, 24);
	File.Write(NumOfFileRecords, 24);
/*	File.Write(ColorIsWidthUnite, 24);
	File.Write(Palette.FindElement(ColorIsWidthUnite), 20);
	File.Write(ColorIsHeightUnite, 24);
	File.Write(Palette.FindElement(ColorIsHeightUnite), 20);
	File.Write(ColorIsRGBASwitch, 24);
	File.Write(Palette.FindElement(ColorIsRGBASwitch), 20);*/
	File.Write(ColorIsWidthUnite, 24);
   tmp1=Palette.FindElement(ColorIsWidthUnite);
	File.Write(tmp1, 20);
	File.Write(ColorIsHeightUnite, 24);
	tmp1=Palette.FindElement(ColorIsHeightUnite);
	File.Write(tmp1, 20);
	File.Write(ColorIsRGBASwitch, 24);
   tmp1=Palette.FindElement(ColorIsRGBASwitch);
	File.Write(tmp1, 20);
	for(tmp2=0; tmp2<4; tmp2++)
		for(tmp1=0; tmp1<Diff[tmp2]; tmp1++)
			File.Write(ChangedTones[tmp2][tmp1], 8);
//----CodeSizes----
	for(tmp1=0; tmp1<=NumOfDiffColors; tmp1++)
		File.Write(Palette[tmp1].Frequency, 20);
//----CodeSizes----
	for(tmp1=0; tmp1<=NumOfDiffColors; tmp1++) Used[Palette.Links[tmp1]]=MaxLong1;
	for(tmp1=0; tmp1<=NumOfFileRecords; tmp1++)
	{
		if(ImageInArray[tmp1]==MaxLong1)
		{
			tmp2=Palette.FindElement(ColorIsWidthUnite);
			File.Write(Palette[tmp2].Code, Palette[tmp2].CodeLength);
		}
		else
		{
			if(ImageInArray[tmp1]==MaxLong2)
			{
				tmp2=Palette.FindElement(ColorIsHeightUnite);
				File.Write(Palette[tmp2].Code, Palette[tmp2].CodeLength);
			}
			else
			{
				tmp2=ImageInArray[tmp1];
				tmp2=Palette.FindElement(tmp2);
				tmp4=Palette.ReLinks[tmp2];
				if(tmp4>NumOfDiffColors)
				{
					tmp2=Palette.FindElement(ColorIsRGBASwitch);
					File.Write(Palette[tmp2].Code, Palette[tmp2].CodeLength);
					RecodeColor(ImageInArray[tmp1], tmp);
					for(tmp3=0; tmp3<4; tmp3++) File.Write(Tones[tmp3][tmp[tmp3]], BP[tmp3]);
				}
				else
				{
					File.Write(Palette[tmp2].Code, Palette[tmp2].CodeLength);
					if(Used[tmp2]==MaxLong1)
					{
						RecodeColor(ImageInArray[tmp1], tmp);
						for(tmp3=0; tmp3<4; tmp3++) File.Write(Tones[tmp3][tmp[tmp3]], BP[tmp3]);
						Used[tmp2]=0;
					}
				}
			}
		}
	}
	File.SaveToFile(FileName);
}
///////////////////////////////////////////////////////////
void RCImage::SimulateSave()
{
	WriteToArray();
	ColorIsWidthUnite=MaxLong1;
	ColorIsHeightUnite=MaxLong2;
	PaintToImage();
}
///////////////////////////////////////////////////////////
void RCImage::SaveAsBitmap(char* FileName)
{
	SetCursor(LoadCursor(NULL,IDC_WAIT));

	WORD x=MaxXCoord+1, y=MaxYCoord+1;
	BYTE Appendix=(4-((x*3)%4))%4;
	DWORD FullRowLen=x*3+Appendix;
	DWORD Size=FullRowLen*y;
	DWORD Color;
	HGLOBAL block=GlobalAlloc(0,Size);
/*	if(block==NULL)
	{
		DANGMESSAGE(" !!!");
		return;
	}*/
	BYTE *Dump;
	Dump=(BYTE*)GlobalLock(block);

	for(y=MaxYCoord; y>=0; y--)
	{
		for(x=0; x<=MaxYCoord; x++)
		{
			Color=Image[x][y];
			Dump[(y-MaxYCoord)*FullRowLen+x*3+2]=Color&0xff;
			Color>>=8;
			Dump[(y-MaxYCoord)*FullRowLen+x*3+1]=Color&0xff;
			Color>>=8;
			Dump[(y-MaxYCoord)*FullRowLen+x*3+0]=Color&0xff;
		}
		for(x=0; x<Appendix; x++)
			Dump[(y-MaxYCoord)*FullRowLen+(MaxXCoord+1)*3+x]=0;
	}
	
	HANDLE _BMP;	
	BITMAPFILEHEADER _BMP_Header;
	BITMAPINFO _BMP_Info;
	BITMAPINFOHEADER _BMP_Info_Header;
	DWORD temp;

	_BMP_Header.bfType=0x4D42;
	_BMP_Header.bfSize=Size+54;
	_BMP_Header.bfReserved1=_BMP_Header.bfReserved2=0;
	_BMP_Header.bfOffBits=54;
	_BMP_Info_Header.biSize=sizeof(BITMAPINFOHEADER);
	_BMP_Info_Header.biWidth=MaxXCoord+1;
	_BMP_Info_Header.biHeight=MaxYCoord+1;
	_BMP_Info_Header.biPlanes=1;
	_BMP_Info_Header.biBitCount=24;
	_BMP_Info_Header.biCompression=BI_RGB;
	_BMP_Info_Header.biSizeImage=Size;
	_BMP_Info_Header.biXPelsPerMeter=_BMP_Info_Header.biYPelsPerMeter=0;
	_BMP_Info_Header.biClrUsed=0;
	_BMP_Info_Header.biClrImportant=0;
	_BMP_Info.bmiHeader=_BMP_Info_Header;
	_BMP=CreateFile(FileName, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0);

	if (_BMP==INVALID_HANDLE_VALUE) return;
	WriteFile(_BMP, &_BMP_Header, sizeof(BITMAPFILEHEADER), &temp, 0);
	WriteFile(_BMP, &_BMP_Info_Header, sizeof(BITMAPINFOHEADER), &temp, 0);
	WriteFile(_BMP, Dump, Size, &temp,0);
	CloseHandle(_BMP);
	GlobalUnlock(block);
	GlobalFree(block);
	SetCursor(LoadCursor(NULL,IDC_ARROW));
}
///////////////////////////////////////////////////////////
void SaveArrayAsBitmap(DWORD **Image, WORD MaxXCoord, WORD MaxYCoord, char* FileName)
{
	SetCursor(LoadCursor(NULL,IDC_WAIT));

	WORD x=MaxXCoord+1, y=MaxYCoord+1;
	BYTE Appendix=(4-((x*3)%4))%4;
	DWORD FullRowLen=x*3+Appendix;
	DWORD Size=FullRowLen*y;
	DWORD Color;
	HGLOBAL block=GlobalAlloc(0,Size);
/*	if(block==NULL)
	{
		DANGMESSAGE(" !!!");
		return;
	}*/
	BYTE *Dump;
	Dump=(BYTE*)GlobalLock(block);

	for(y=MaxYCoord; y>=0; y--)
	{
		for(x=0; x<=MaxYCoord; x++)
		{
			Color=Image[x][y];
			Dump[(y-MaxYCoord)*FullRowLen+x*3+2]=Color&0xff;
			Color>>=8;
			Dump[(y-MaxYCoord)*FullRowLen+x*3+1]=Color&0xff;
			Color>>=8;
			Dump[(y-MaxYCoord)*FullRowLen+x*3+0]=Color&0xff;
		}
		for(x=0; x<Appendix; x++)
			Dump[(y-MaxYCoord)*FullRowLen+(MaxXCoord+1)*3+x]=0;
	}
	
	HANDLE _BMP;	
	BITMAPFILEHEADER _BMP_Header;
	BITMAPINFO _BMP_Info;
	BITMAPINFOHEADER _BMP_Info_Header;
	DWORD temp;

	_BMP_Header.bfType=0x4D42;
	_BMP_Header.bfSize=Size+54;
	_BMP_Header.bfReserved1=_BMP_Header.bfReserved2=0;
	_BMP_Header.bfOffBits=54;
	_BMP_Info_Header.biSize=sizeof(BITMAPINFOHEADER);
	_BMP_Info_Header.biWidth=MaxXCoord+1;
	_BMP_Info_Header.biHeight=MaxYCoord+1;
	_BMP_Info_Header.biPlanes=1;
	_BMP_Info_Header.biBitCount=24;
	_BMP_Info_Header.biCompression=BI_RGB;
	_BMP_Info_Header.biSizeImage=Size;
	_BMP_Info_Header.biXPelsPerMeter=_BMP_Info_Header.biYPelsPerMeter=0;
	_BMP_Info_Header.biClrUsed=0;
	_BMP_Info_Header.biClrImportant=0;
	_BMP_Info.bmiHeader=_BMP_Info_Header;
	_BMP=CreateFile(FileName, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0);

	if (_BMP==INVALID_HANDLE_VALUE) return;
	WriteFile(_BMP, &_BMP_Header, sizeof(BITMAPFILEHEADER), &temp, 0);
	WriteFile(_BMP, &_BMP_Info_Header, sizeof(BITMAPINFOHEADER), &temp, 0);
	WriteFile(_BMP, Dump, Size, &temp,0);
	CloseHandle(_BMP);
	GlobalUnlock(block);
	GlobalFree(block);
	SetCursor(LoadCursor(NULL,IDC_ARROW));
}
///////////////////////////////////////////////////////////

#endif
