unit FtpMsg;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, RichEdit,
   StdCtrls, ComCtrls, Ftp;

{$I mftp.inc}

type
   TMessengerStyle = (msgClassic, msgMFtp, msgMFtpTime);
   TMessageType  = (msgCommand, msgError, msgException, msgReply, msgStatus);

   TMessengerShowEvent = procedure (Sender: TObject; info: FtpInfo; var Show: Boolean) of object;
   TMessengerShowEventE = procedure (Sender: TObject; error: FtpError; var Show: Boolean) of object;

   TMFtpMessenger = class(TCustomRichEdit)
   private
      FFtp:                      TMFtp;

      FStyle:                    TMessengerStyle;
      FShowEvent:                TMessengerShowEvent;
      FShowEventE:               TMessengerShowEventE;

      ShowDisconnect:            Boolean;

      HOnFtpInfo:                Integer;
      HOnFtpError:               Integer;

      procedure NewOnFtpInfo(Sender: TObject; info: FtpInfo; addinfo: String);
      procedure NewOnFtpError(Sender: TObject; error: FtpError; addinfo: String);

      procedure SetClient(NewFtp: TMFtp);
      procedure SetStyle(NewStyle: TMessengerStyle);
   protected
   public
      constructor Create(AOwner: TComponent); override;

      procedure AddMessage(Msg: String; MsgType: TMessageType);
   published
      property Client: TMFtp read FFtp write SetClient;
      property Style: TMessengerStyle read FStyle write SetStyle;

      property OnErrorShow: TMessengerShowEventE read FShowEventE write FShowEventE;
      property OnMessageShow: TMessengerShowEvent read FShowEvent write FShowEvent;

      property Align;
      property HideScrollBars;
      property HideSelection;
      property PopupMenu;
      property ScrollBars;
      property WordWrap;

      property OnClick;
      property OnDblClick;
      property OnEnter;
      property OnExit;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;

      property Anchors;
      property BiDiMode;
      property BorderWidth;
      property Constraints;
      property DragKind;
      property DragMode;
      property ParentBiDiMode;

      property OnEndDock;
      property OnMouseWheel;
      property OnMouseWheelDown;
      property OnMouseWheelUp;
      property OnStartDock;
   end;

implementation

constructor TMFtpMessenger.Create;
begin
   inherited Create(AOwner);

   ImeMode := imClose;
   ReadOnly := True;
   WordWrap := False;

   ShowDisconnect := False;
end;

procedure TMFtpMessenger.SetClient;
begin
   if FFtp = NewFtp then Exit;

   if Assigned(FFtp) then
   begin
      with FFtp do
      begin
         UnRegisterInfoEvent(HOnFtpInfo);
         UnRegisterInfoEvent(HOnFtpError);
      end;
   end;

   FFtp := NewFtp;

   if not Assigned(FFtp) then Exit;

   with FFtp do
   begin
      HOnFtpInfo := RegisterInfoEvent(NewOnFtpInfo);
      HOnFtpError := RegisterErrorEvent(NewOnFtpError);
   end;
end;

procedure TMFtpMessenger.SetStyle;
begin
   FStyle := NewStyle;

   if FStyle = msgClassic then
      Font.Name := 'Courier New'
   else
      Font.Name := 'MS Sans Serif';
end;

procedure TMFtpMessenger.AddMessage;
var C: TColor;
    P: String;
    i: Integer;
begin
   if Trim(Msg) = '' then Exit;

   C := clBlack;    {to make compiler happy :-)}
   if FStyle = msgClassic then
   begin
      case MsgType of
         msgCommand:
         begin
            C := clGreen;
            P := msgCommandx;
         end;
         msgError:
         begin
            C := clRed;
            P := msgErrorx;
         end;
         msgException:
         begin
            C := clRed;
            P := msgExceptionx;
         end;
         msgReply:
         begin
            //C := clBlack;
            P := msgReplyx;
         end;
         msgStatus:
         begin
            C := clBlue;
            P := msgStatusx;
         end;
      end;

      SelStart := -1;
      SelAttributes.Color := C;
      Msg := P + Msg;
      i := Pos(#13#10, Msg);
      if i = 0 then
      begin
         Lines.Add(Msg)
      end
      else
      begin
         Lines.Add(Copy(Msg, 1, i - 1));
         Delete(Msg, 1, i + 1);
         i := Pos(#13#10, Msg);

         while i <> 0 do
         begin
            Lines.Add('            ' + Copy(Msg, 1, i - 1));
            Delete(Msg, 1, i + 1);
            i := Pos(#13#10, Msg);
         end;

         Lines.Add('            ' + Msg)
      end;
   end
   else
   begin
      case MsgType of
         msgCommand:
         begin
            C := clBlue;
         end;
         msgError:
         begin
            C := clRed;
         end;
         msgException:
         begin
            C := clRed;
         end;
         msgReply:
         begin
            C := clGreen;
         end;
         msgStatus:
         begin
            C := clPurple;
         end;
      end;

      SelStart := -1;
      SelAttributes.Color := C;

      if (Style = msgMFtpTime) and (MsgType <> msgReply) then
         Msg:='[' + DateTimetoStr(Now) + '] ' + Msg;
      Lines.Add(Msg);
   end;
end;

procedure TMFtpMessenger.NewOnFtpInfo;
var Show: Boolean;
begin
   if Assigned(FShowEvent) then
   begin
      Show := True;
      FShowEvent(Self, info, Show);
      if not Show then Exit;
   end;

   case info of
      ftpServerConnected:
      begin
         with FFtp do
         begin
            if (Server = '') and (addinfo = '') then
               AddMessage(msgConnectTo + msgSpecifiedS + msgWelcome, msgStatus)
            else
            if (Server <> '') and (Server <> addinfo) then
            begin
               if addinfo <> '' then
                  AddMessage(msgConnectTo + Server + '(ip: ' + addinfo + ')' + msgWelcome, msgStatus)
               else
                  AddMessage(msgConnectTo + Server + msgWelcome, msgStatus);
            end
            else
            begin
               AddMessage(msgConnectTo + addinfo + msgWelcome, msgStatus);
            end;
         end;
         ShowDisconnect := True;
      end;
      ftpTraceIn: AddMessage(addinfo, msgReply);
      ftpTraceOut: AddMessage(addinfo, msgCommand);
      ftpRetrying: AddMessage(addinfo + #13#10, msgStatus);
      ftpNotSupportResume: AddMessage(addinfo + #13#10, msgStatus);
      ftpSupportResume: AddMessage(addinfo + #13#10, msgStatus);
      ftpServerDisconnected:
      begin
         if ShowDisconnect then
         begin
            AddMessage(msgDisconnected + #13#10, msgStatus);
            ShowDisconnect := False;
         end;
      end;
      ftpNothing: AddMessage(addinfo + #13#10, msgStatus);
      ftpLoggedIn: AddMessage(msgLoginS + #13#10, msgStatus);
   end;
end;

procedure TMFtpMessenger.NewOnFtpError;
var Show: Boolean;
begin
   if Assigned(FShowEventE) then
   begin
      Show := True;
      FShowEventE(Self, error, Show);
      if not Show then Exit;
   end;

   AddMessage(addinfo + #13#10, msgError);
end;

end.
