unit FtpSearch;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
   WinINet, FtpData;

{$I mftp.inc}

const
   SearchTypeStr: Array[0..10] of String = (
      'Case+insensitive+substring+search',
      'Case+sensitive+substring+search',
      'Case+insensitive+glob+search',
      'Case+sensitive+glob+search',
      'Regular+expression+search',
      'Exact+search',
      'Case+insensitive+substring+match',
      'Case+sensitive+substring+match',
      'Case+insensitive+glob+match',
      'Case+sensitive+glob+match',
      'Regular+expression+match');

   BUFFER_SIZE = 1024;

type
   { C = case, I = insensitive, N = sensitive,
     G = glob, E = Exact, X = expression,
     S = search, M = match, R = Regular,
     U = substring }

   TMFtpSearchType = (mstCIUS, mstCNUS, mstCIGS, mstCNGS,
                      mstRXS, mstES, mstCIUM, mstCNUM,
                      mstCIGM, mstCNGM, mstEXM);

   TMFtpSearchInfo = (searchStart, searchDone, searchFileFound);
   TMFtpSearchInfoEvent = procedure (Sender: TObject; info: TMFtpSearchInfo; addinfo: String) of object;

   TMFtpSearch = class(TComponent)
   private
      FBusy:                     Boolean;

      FServer:                   TStrings;

      FCache:                    Boolean;
      FProxy:                    Boolean;

      FFile:                     String;
      FFiles:                    TMFtpFileInfoList;

      FMaxHit:                   Longword;
      FMaxMatch:                 Longword;
      FType:                     TMFtpSearchType;

      FSearchInfoEvent:          TMFtpSearchInfoEvent;

      function BuildURL: String;

      procedure Parse(L: TStringList);
      procedure SearchMain(SServer: String; SPort: Integer);

      procedure SetServer(S: TStrings);
   public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

      procedure Search;

      property Busy: Boolean read FBusy;
      property Files: TMFtpFileInfoList read FFiles;
   published
      property Server: TStrings read FServer write SetServer;

      property Cache: Boolean read FCache write FCache;
      property Proxy: Boolean read FProxy write FProxy;

      property Filename: String read FFile write FFile;      

      property MaxHit: Longword read FMaxHit write FMaxHit;
      property MaxMatch: Longword read FMaxMatch write FMaxMatch;
      property SearchType: TMFtpSearchType read FType write FType;

      property OnSearchInfo: TMFtpSearchInfoEvent read FSearchInfoEvent write FSearchInfoEvent;
   end;

implementation

constructor TMFtpSearch.Create;
begin
   inherited Create(AOwner);

   FServer := TStringList.Create;
   FServer.Add('ftpsearch.lycos.com:80');

   FFiles := TMFtpFileInfoList.Create;

   FCache := True;

   FMaxHit := 200;
   FMaxMatch := 200;
end;

destructor TMFtpSearch.Destroy;
begin
   FServer.Free;
   FFiles.MyFree;

   inherited Destroy;
end;

function TMFtpSearch.BuildURL;
begin
   Result := '/ftpsearch?query=' + Trim(FFile) +
             '&hits=' + IntToStr(FMaxHit) + '&matches=' + IntToStr(FMaxMatch) +
             '&doit=Search&f1=Count&f2=Mode&f3=Len&f4=Date&f5=Time&f6=Name&' +
             'form=medium';
end;

procedure TMFtpSearch.SetServer;
begin
   FServer.Assign(S);
end;

procedure TMFtpSearch.Parse;
var Attrib, Size, DT, URL, S: String;
    i, j, j1: Integer;
begin
   for i := 0 to L.Count - 1 do
   begin
      S := Trim(L[i]);

      j := Pos('<TT>', S);
      if j > 0 then
      begin
         j1 := Pos('</TT>', S);
         Attrib := Trim(Copy(S, j + 4, j1 - j - 4));
         Delete(S, 1, j1 + 5);

         j := Pos('<B>', S );
         j1 := Pos('</B>', S);
         Size := Trim(Copy(S, j + 3, j1 - j - 3));
         Delete(S, 1, j1 + 4);

         j := Pos('<A HREF="', S);
         DT := Trim(Copy(S, 1, j - 1));
         Delete(S, 1, j + 8);

         j := Pos('">', S);
         URL := Trim(Copy(S, 1, j - 1));

         // work arounds here
         if Copy(URL, 1, 6) <> 'ftp://' then
         begin
            j := Pos('oquery=', URL);
            URL := 'ftp://' + Copy(URL, j + 7, 9999);
         end;

         j := Pos('type=Navigate&query=', URL);

         if j > 0 then
         begin
            URL := 'ftp://' + Copy(URL, j + 20, 9999);

            j := Pos('%2', URL);
            while j > 0 do
            begin
               if URL[j + 2] = 'e' then URL := Copy(URL, 1, j - 1) + '.' + Copy(URL, j + 3, 9999) else
               if URL[j + 2] = 'f' then URL := Copy(URL, 1, j - 1) + '/' + Copy(URL, j + 3, 9999);
               j := Pos('%2', URL);
            end;
         end;

         if FFiles.IndexOf(URL) < 0 then
         begin
            {$ifdef ENHANCED_LISTING_PARSING}
            FFiles.Add(URL, Attrib, DT, Size, '', '', '','');
            {$else}
            FFiles.Add(URL, Attrib, DT, Size, '', '');
            {$endif}

            if Assigned(FSearchInfoEvent) then FSearchInfoEvent(Self, searchFileFound, URL);
         end;
      end;

      if i mod 25 = 0 then Application.ProcessMessages;
   end;
end;

// to simplify the search routine, we won't check error for it's not
// necessary for us
procedure TMFtpSearch.SearchMain;
var FHandle, FHttpHandle: HINTERNET;
    TS: TStringList;
    FBufs: String;
    FBuf: PChar;
    Len: DWORD;
begin
   if FProxy then
      FHandle := InternetOpen('MFtpINet', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0)
   else
      FHandle := InternetOpen('MFtpINet', INTERNET_OPEN_TYPE_DIRECT, nil, nil, 0);

   FHttpHandle := InternetConnect(FHandle, PChar(SServer), SPort, '', '',
                                  INTERNET_SERVICE_HTTP, 0, 0);
   Application.ProcessMessages;

   if FCache then
      FHttpHandle := HttpOpenRequest(FHttpHandle, nil, PChar(BuildURL), nil, nil, nil, 0, 0)
   else
      FHttpHandle := HttpOpenRequest(FHttpHandle, nil, PChar(BuildURL), nil, nil, nil,
                     INTERNET_FLAG_RELOAD, 0);

   HttpSendRequest(FHttpHandle, nil, 0, nil, 0);
   Application.ProcessMessages;

   FBuf := StrAlloc(BUFFER_SIZE + 1);
   repeat
      InternetReadFile(FHttpHandle, FBuf, BUFFER_SIZE, Len);
      FBuf[Len] := Char(0);
      FBufS := FBufS + StrPas(FBuf);

      Application.ProcessMessages;
   until (Len < BUFFER_SIZE);

   TS := TStringList.Create;
   try
      TS.Text := FBufS;
      Parse(TS);
   finally
      TS.Free;
   end;

   FBufs := '';
   StrDispose(FBuf);

   InternetCloseHandle(FHttpHandle);
   InternetCloseHandle(FHandle);
end;

procedure TMFtpSearch.Search;
var i, j: Integer;
begin
   if FBusy then Exit;
   if Trim(FFile) = '' then Exit;

   FBusy := True;
   FFiles.Clear;

   for i := 0 to FServer.Count - 1 do
   begin
      FServer[i] := Trim(FServer[i]);

      if Trim(FServer[i]) <> '' then
      begin
         if Assigned(FSearchInfoEvent) then FSearchInfoEvent(Self, searchStart, FServer[i]);

         j := Pos(':', FServer[i]);

         if j > 0 then
            SearchMain(Copy(FServer[i], 1, j - 1), StrToInt(Copy(FServer[i], j + 1, 999)))
         else
            SearchMain(Copy(FServer[i], 1, j - 1), 80);

         if Assigned(FSearchInfoEvent) then FSearchInfoEvent(Self, searchDone, FServer[i]);
      end;
   end;

   FBusy := False;
end;

end.

