Unit TorDate;

Interface

{***********************************************************}
{*                                                         *}
{*  TORDATE.PAS                                            *}
{*                                                         *}
{*  Copyright (c) Konstantin Klyagin, 1998.                *}
{*  Date manipulation functions (Tornado Development Kit)  *}
{*                                                         *}
{***********************************************************}

(*
    Note: Date parameter must be in MM-DD-YY format,
    Long2Date function returns date in such format, too.
*)

Function Date2Long (Date: String) : LongInt;
Function Long2Date (Julian : LongInt): String;

Implementation

Const
  BadDate  = $FFFF;
  Threshold2000 : Integer = 1900;
  MinYear  = 1900;
  MaxYear  = 2078;
  MinDate  = $0000;
  MaxDate  = $FF62;
  Date1900 = $0000;
  Date1980 = $7223;
  Date2000 = $8EAC;
  First2Months = 58;

Function IsLeapYear (Year : Integer) : Boolean;
Begin
  IsLeapYear := (Year Mod 4 = 0) And (Year Mod 4000 <> 0) And
  ( (Year Mod 100 <> 0) Or (Year Mod 400 = 0) );
End;

Function DaysInMonth (Month, Year : Integer) : Integer;
Begin
  If Word (Year) < 100 Then Begin
    Inc (Year, 1900);
    If Year < Threshold2000 Then
      Inc (Year, 100);
  End;
  Case Month Of
    1, 3, 5, 7, 8, 10, 12 :
                             DaysInMonth := 31;
    4, 6, 9, 11 :
                   DaysInMonth := 30;
    2 :
         DaysInMonth := 28 + Ord (IsLeapYear (Year) );
    Else
      DaysInMonth := 0;
  End;
End;

Function ValidDate (Day, Month, Year : Integer) : Boolean;
Begin
  If Word (Year) < 100 Then Begin
    Inc (Year, 1900);
    If Year < Threshold2000 Then Inc (Year, 100);
  End;
  If (Day < 1) Or (Year < MinYear) Or (Year > MaxYear) Then
    ValidDate := False
  Else Case Month Of
    1..12 :
             ValidDate := Day <= DaysInMonth (Month, Year);
    Else
      ValidDate := False;
  End
End;

Function Date2Long (Date: String) : LongInt;
Var
  Day, Month, Year, Err: {$IFNDEF OS2} Word; {$ELSE} LongInt; {$ENDIF}

Begin
  Insert ('19', Date, 7);
  Val (Copy (Date, 4, 2), Day, Err);
  Val (Copy (Date, 1, 2), Month, Err);
  Val (Copy (Date, Length (Date)-3, 4), Year, Err);

  If Word (Year) < 100 Then
  Begin
    Inc (Year, 1900);
    If Year < Threshold2000 Then Inc (Year, 100);
  End;

  If Not ValidDate (Day, Month, Year) Then Date2Long := BadDate
  Else If (Year = MinYear) And (Month < 3) Then
    If Month = 1 Then Date2Long := Pred (Day) Else Date2Long := Day + 30
  Else Begin
    If Month > 2 Then Dec (Month, 3)
    Else Begin
      Inc (Month, 9);
      Dec (Year);
    End;
    Dec (Year, MinYear);
    Date2Long := ((LongInt (Year) * 1461) Div 4) +
                 (((153 * Month) + 2) Div 5) +
                    Day + First2Months;
  End;
End;

Function Long2Date (Julian : LongInt): String;
Var
  I, J                  : LongInt;
  Day, Month, Year      : Integer;
  S, S1                 : String;

Begin
  If Julian = BadDate Then
  Begin
    Long2Date := 'BAD DATE';
    Exit;
  End Else
  If Julian <= First2Months Then
  Begin
    Year := MinYear;
    If Julian <= 30 Then
    Begin
      Month := 1;
      Day := Succ (Julian);
    End Else
    Begin
      Month := 2;
      Day := Julian - 30;
    End;
  End Else
  Begin
    I := (4 * LongInt (Julian - First2Months) ) - 1;
    Year := I Div 1461;                         {!!.09}
    I := (5 * ( (I Mod 1461) Div 4) ) + 2;          {!!.09}
    Month := I Div 153;
    Day := ( (I Mod 153) + 5) Div 5;
    If Month < 10 Then Inc (Month, 3) Else
    Begin
      Dec (Month, 9);
      Inc (Year);
    End;
    Inc (Year, MinYear);
  End;

  Str (Month, S); If Length (S) < 2 Then S := '0' + S;
  Str (Day, S1); If Length (S1) < 2 Then S1 := '0' + S1; S := S + '-' + S1;
  Str (Year, S1); S := S + '-' + Copy (S1, 3, 2);
  Long2Date := S;
End;

End.